/*------------------------------------------------------------------------
   (C) Copyright Marxmeier Software AG 2002,2003
   
   Project:  HP ELOQUENCE
   Module:   dll
   File:     dll.h
   Function: DLL header file

   This software and documentation is based in part on HP software
   and documentation under license from Hewlett-Packard Company.
   HP is a trademark of Hewlett-Packard Company.
   Copyright 2002 Hewlett-Packard Company.

   Edit history:
   10.11.1990 - creation
   21.02.2003 - LP64 clean
   26.07.2003 - HP-UX ILP32/LP64 interop
   
------------------------------------------------------------------------*/

/* @(#)$Revision: 24.2 $ */

#ifndef _DLL_INCLUDED
#define _DLL_INCLUDED

/*------------------------------------------------------------------------
      DLL public
------------------------------------------------------------------------*/

typedef enum {
   DLL_BAD_ARG = 0,
   DLL_INTEGER,
   DLL_REAL,
   DLL_STRING
} t_DLL_TYPE;

typedef struct {
   t_DLL_TYPE type;  /* argument type */
   int pass_by_ref;  /* pass by reference flag */
   void *ptr;        /* argument pointer (4 byte aligned) */
   int maxlen;       /* maximum string length */
   int elsz;         /* size of element (in 4 byte units) */
   int elcnt;        /* number of elements (>1 if array) */
} t_DLL_ARG;

typedef struct {
   int len;          /* string length */
   char s[1];        /* string data */
} t_DLL_string;


/*------------------------------------------------------------------------
      DLL private
------------------------------------------------------------------------*/

#ifdef _DLL_PRIVATE

#define DLL_MAGIC_SZ 8
#define DLL_MAGIC    "EQ.DLL\0"
#define DLL_VERSION  0x0101

typedef struct {
   t_DLL_TYPE type;  /* argument type */
   int pass_by_ref;  /* pass by reference flag */
#if defined(__hpux) && defined(__LP64__)
   u_int32_t ptr;    /* ouch! */
#else
   void *ptr;        /* argument pointer (4 byte aligned) */
#endif
   int maxlen;       /* maximum string length */
   int elsz;         /* size of element (in 4 byte units) */
   int elcnt;        /* number of elements (>1 if array) */
} t_DLL_ARG_COMPAT;

typedef struct {
   char magic[DLL_MAGIC_SZ];     /* magic constant */
   int version;                  /* version number */
   int pid;                      /* parent process id */
#if defined(_WIN32)
   HANDLE hExitEvent;		 /* Exit Event object */
   HANDLE hParentEvent;		 /* event object: signal parent process */
   HANDLE hDLLEvent;		    /* event object: signal DLL process */
#endif
#if !defined(__hpux)
   void *baseaddr;          /* shared memory base address */
#endif
   int error;                    /* error number */
   int shmsz;                    /* shm memory size */
#if defined(__hpux) && defined(__LP64__)
   u_int32_t memp;               /* ouch! */
#else
   void *memp;                   /* pointer past last argument */
#endif
   int fn;                       /* function no */
   int argc;                     /* number of args */
   t_DLL_ARG_COMPAT arg[1];      /* arguments ... */
} t_DLL_INFO;

#endif   /* _DLL_PRIVATE */

/*------------------------------------------------------------------------
      DLL prototypes
------------------------------------------------------------------------*/

#ifndef _NO_DLL_PROTOTYPES

#ifdef __STDC__
# define _p1(a) a
#else
# define _p1(a)
#endif

void *u_get_baseptr(_p1(void));
int u_get_argc(_p1(void));
t_DLL_ARG u_get_arg(_p1(int));
int u_unref_arg(_p1(int));

void dll_setup(_p1(void));
void dll_cleanup(_p1(void));
int dll_main(_p1(void));

#undef _p1

#endif   /* _NO_DLL_PROTOTYPES */
#endif   /* _DLL_INCLUDED */
