/* log.c

   This dll will write given arguments to log file.
   It also demonstrates the usage of variable arguments.
*/

/* --- includes --- */

#include <stdio.h>
#ifdef __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#ifndef DLLCC
#include <dllif.h>
#endif

/* --- module globals --- */

FILE *log_fp;

struct {
   enum EqArgType type;
   char *txt;
} arg_type_names[] = {
   { EqArg_VarArg,         "VarArg"          },    /* internal only */
   { EqArg_IntByValue,     "IntByValue"      },    /* unused */
   { EqArg_IntByReference, "IntByReference"  },
   { EqArg_IntArray,       "IntArray"        },
   { EqArg_RealByValue,    "RealByValue"     },    /* unused */
   { EqArg_RealByReference,"RealByReference" },
   { EqArg_RealArray,      "RealArray"       },
   { EqArg_zString,        "zString"         },    /* unused */
   { EqArg_String,         "String"          },
   { EqArg_StringArray,    "StringArray"     },
   { -1,                   "Unknown"         }     /* end-of-list */
};


/* --- (re-)open log file --- */

#ifdef __STDC__
EqDLL open( EqChar *name )
#else
EqDLL open( name )
EqChar *name;
#endif
{
   if(log_fp != NULL)            /* if file already open ? */
      fclose(log_fp);            /* -> close former one    */

   log_fp = fopen(name, "w");    /* open new file          */
   if(log_fp == NULL)            /* failed ?               */
      return(2000);              /* -> error               */

   return(0);                    /* success                */
}

/* --- write data to log file --- */

/*
   NOTE:

   1. c compliler requires a fixed argument
   2. all varargs are given "pass-by-reference".
      HP ELOQUENCE variables will only be changed if it's really
      a "pass by reference".
*/

#ifdef __STDC__
EqDLL log( EqChar *text, ... )
#else
EqDLL log( text, va_alist )
EqChar *text;
va_dcl
#endif
{
   va_list args;
   int i;
   struct EqFuncInfo *f_info;

   if(log_fp == NULL)            /* log file opened ?    */
      return(2001);              /* -> error             */

                                 /* write initial text   */
   fprintf(log_fp, "\n*** %s\n", text);
   
   f_info = EqFuncInfo();        /* get information about function */
                                 /* (number of arguments)          */

#ifdef __STDC__
   va_start(args, text);         /* begin vararg processing        */
#else
   va_start(args);
#endif
                                 /* NOTE: 1st argument is 'text'   */
                                 /*       so it's skipped below    */
   for(i = 1; i < f_info->arg_cnt; i++)
   {
      struct EqArgInfo *a_info;
      void *arg;
   
      EqString *eq;
      int *ip, e;
      double *dp;

      arg = va_arg(args, void *);   /* get pointer to argument */

      a_info = EqArgInfo(arg);      /* get info about argument */

                                    /* output argument type */
      for(e = 0; arg_type_names[e].type != -1; e++)
        if(arg_type_names[e].type == a_info->type)
           break;

      fprintf(log_fp, "Argument Type is %s\n", arg_type_names[e].txt);

      for(e = 0; e < a_info->elcnt; e++)
      {
         if(e)
            fprintf(log_fp, "\n");
         if(a_info->elcnt > 1)
            fprintf(log_fp, "[%2d] ", e);

         switch( a_info->type )
         {
            case EqArg_IntByReference:          /* integer argument */
            case EqArg_IntArray:
               ip = (int *)arg;
               fprintf(log_fp, "%d", ip[e]);
               break;

            case EqArg_RealByReference:         /* real argument */
            case EqArg_RealArray:
               dp = (double *)arg;
               fprintf(log_fp, "%1.12g", dp[e]); 
               break;

            case EqArg_String:                  /* string argument */
            case EqArg_StringArray:
               eq = (EqString *)arg;
               fprintf(log_fp, "\"%.*s\"", eq[e].l, eq[e].p);
               break;

            default:                            /* should never happen */
               fprintf(log_fp, "*ERROR*");
         }
      }

      fprintf(log_fp, "\n");
   }

   va_end(args);
   return(0);
}
